package client

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"
	"net/http"
)

func (r *Remote) send(req *http.Request) (*http.Response, error) {
	if req == nil {
		return nil, nil
	}

	if r.secret != "" {
		req.Header.Add("secret", r.secret)
	}

	return r.client.Do(req)
}

func (r *Remote) request(method, path string, body io.Reader) (*http.Response, error) {
	if req, err := http.NewRequest(method, r.URL(path), body); err != nil {
		return nil, err
	} else {
		return r.send(req)
	}
}

func (r *Remote) requestNoResp(method, path string, body io.Reader) error {
	if resp, err := r.request(method, path, body); err != nil {
		return err
	} else {
		return resp.Body.Close()
	}
}

func (r *Remote) requestJSON(method, path string, v interface{}) (*http.Response, error) {
	var reader *bytes.Reader
	if v != nil {
		if payload, err := json.Marshal(v); err != nil {
			return nil, err
		} else {
			reader = bytes.NewReader(payload)
		}
	}
	return r.request(method, path, reader)
}

func (r *Remote) requestJSONnoResp(method, path string, v interface{}) error {
	if resp, err := r.requestJSON(method, path, v); err != nil {
		return err
	} else {
		return resp.Body.Close()
	}
}

func (r *Remote) fetch(method, path string, v interface{}, body io.Reader) error {
	if resp, err := r.request(method, path, body); err != nil {
		return err
	} else {
		if v == nil {
			return nil
		}
		if err = unmarshal(resp.Body, v); err != nil {
			return err
		}
	}
	return nil
}

func (r *Remote) fetchAll(method, path string, body io.Reader) ([]byte, error) {
	if resp, err := r.request(method, path, body); err != nil {
		return nil, err
	} else {
		defer func() {
			if err = resp.Body.Close(); err != nil {
				fmt.Printf("Error closing body after reading all, %s\n", err)
			}
		}()
		var content []byte
		if content, err = io.ReadAll(resp.Body); err != nil {
			return nil, err
		} else {
			return content, nil
		}
	}
}

func unmarshal(reader io.ReadCloser, v interface{}) error {
	defer func() {
		if err := reader.Close(); err != nil {
			fmt.Printf("Error closing reader after unmarshalling, %s\n", err)
		}
	}()
	if err := json.NewDecoder(reader).Decode(v); err != nil {
		return err
	}
	return nil
}
